@@ -10,47 +10,42 @@ from pfhedge.nn.functional import european_binary_payoff
 
 from ..primary.base import BasePrimary
 from .base import BaseDerivative
-from .base import BaseOption
+from .base import OptionMixin
 
 
-class EuropeanBinaryOption(BaseOption):
+class EuropeanBinaryOption(BaseDerivative, OptionMixin):
     r"""European binary option.
 
-    An American binary call option pays an unit amount of cash if and only if
-    the underlying asset's price at maturity is equal or greater than the strike price.
-
-    An American binary put option pays an unit amount of cash if and only if
-    the underlying asset's price at maturity is equal or smaller than the strike price.
-
-    The payoff of an American binary call option is given by:
+    The payoff of a European binary call option is given by
 
     .. math::
         \mathrm{payoff} =
         \begin{cases}
             1 & (S \geq K) \\
             0 & (\text{otherwise})
-        \end{cases}
+        \end{cases} ,
 
-    with :math:`S` being the underlying asset's price at maturity and
-    :math:`K` being the strike price (`strike`) of the option
+    where
+    :math:`S` is the underlier's spot price at maturity and
+    :math:`K` is the strike.
 
-    The payoff of an American binary put option is given by:
+    The payoff of a European binary put option is given by
 
     .. math::
         \mathrm{payoff} =
         \begin{cases}
             1 & (S \leq K) \\
             0 & (\text{otherwise})
-        \end{cases}
+        \end{cases} .
 
     .. seealso::
-        :func:`pfhedge.nn.functional.european_binary_payoff`: Payoff function.
+        - :func:`pfhedge.nn.functional.european_binary_payoff`
 
     Args:
         underlier (:class:`BasePrimary`): The underlying instrument of the option.
         call (bool, default=True): Specifies whether the option is call or put.
         strike (float, default=1): The strike price of the option.
-        maturity (float, default=20/250) The maturity of the option.
+        maturity (float, default=20/250): The maturity of the option.
 
     Attributes:
         dtype (torch.dtype): The dtype with which the simulated time-series are
@@ -82,7 +77,7 @@ class EuropeanBinaryOption(BaseOption):
         device: Optional[torch.device] = None,
     ) -> None:
         super().__init__()
-        self.underlier = underlier
+        self.register_underlier("underlier", underlier)
         self.call = call
         self.strike = strike
         self.maturity = maturity
@@ -115,8 +110,10 @@ _set_attr_and_docstring(EuropeanBinaryOption, "to", BaseDerivative.to)
 _set_attr_and_docstring(EuropeanBinaryOption, "ul", BaseDerivative.ul)
 _set_attr_and_docstring(EuropeanBinaryOption, "list", BaseDerivative.list)
 _set_docstring(EuropeanBinaryOption, "payoff", BaseDerivative.payoff)
-_set_attr_and_docstring(EuropeanBinaryOption, "moneyness", BaseOption.moneyness)
-_set_attr_and_docstring(EuropeanBinaryOption, "log_moneyness", BaseOption.log_moneyness)
+_set_attr_and_docstring(EuropeanBinaryOption, "moneyness", OptionMixin.moneyness)
 _set_attr_and_docstring(
-    EuropeanBinaryOption, "time_to_maturity", BaseOption.time_to_maturity
+    EuropeanBinaryOption, "log_moneyness", OptionMixin.log_moneyness
+)
+_set_attr_and_docstring(
+    EuropeanBinaryOption, "time_to_maturity", OptionMixin.time_to_maturity
 )
