@@ -1,19 +1,21 @@
 from typing import List
+from typing import Optional
 
 import torch
 from torch import Tensor
-from torch.distributions.utils import broadcast_all
 
 from pfhedge._utils.bisect import find_implied_volatility
 from pfhedge._utils.doc import _set_attr_and_docstring
 from pfhedge._utils.doc import _set_docstring
 from pfhedge._utils.str import _format_float
-from pfhedge.nn.functional import d1
-from pfhedge.nn.functional import d2
-from pfhedge.nn.functional import ncdf
-from pfhedge.nn.functional import npdf
+from pfhedge.instruments import EuropeanBinaryOption
+from pfhedge.nn.functional import bs_european_binary_delta
+from pfhedge.nn.functional import bs_european_binary_price
 
 from ._base import BSModuleMixin
+from ._base import acquire_params_from_derivative_0
+from ._base import acquire_params_from_derivative_1
+from .black_scholes import BlackScholesModuleFactory
 
 
 class BSEuropeanBinaryOption(BSModuleMixin):
@@ -58,10 +60,16 @@ class BSEuropeanBinaryOption(BSModuleMixin):
                 [6.1953]])
     """
 
-    def __init__(self, call: bool = True, strike: float = 1.0):
+    def __init__(
+        self,
+        call: bool = True,
+        strike: float = 1.0,
+        derivative: Optional[EuropeanBinaryOption] = None,
+    ):
         super().__init__()
         self.call = call
         self.strike = strike
+        self.derivative = derivative
 
     @classmethod
     def from_derivative(cls, derivative):
@@ -83,7 +91,9 @@ class BSEuropeanBinaryOption(BSModuleMixin):
             >>> m
             BSEuropeanBinaryOption(strike=1.1000)
         """
-        return cls(call=derivative.call, strike=derivative.strike)
+        return cls(
+            call=derivative.call, strike=derivative.strike, derivative=derivative
+        )
 
     def extra_repr(self) -> str:
         params = []
@@ -96,14 +106,17 @@ class BSEuropeanBinaryOption(BSModuleMixin):
         return ["log_moneyness", "time_to_maturity", "volatility"]
 
     def price(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns price of the derivative.
 
         Args:
-            log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional)): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional)): Time to expiry of the option.
+            volatility (torch.Tensor, optional)): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)`
@@ -113,24 +126,37 @@ class BSEuropeanBinaryOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
         """
-        s, t, v = broadcast_all(log_moneyness, time_to_maturity, volatility)
-
-        price = ncdf(d2(s, t, v))
-        price = 1.0 - price if not self.call else price  # put-call parity
-
-        return price
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
+        return bs_european_binary_price(
+            log_moneyness=log_moneyness,
+            time_to_maturity=time_to_maturity,
+            volatility=volatility,
+            call=self.call,
+        )
 
     @torch.enable_grad()
     def delta(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns delta of the derivative.
 
         Args:
-            log_moneyness: (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness: (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)`
@@ -140,24 +166,37 @@ class BSEuropeanBinaryOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            Parameters are not optional if the module has not accepted a derivative in its initialization.
         """
-        s, t, v = broadcast_all(log_moneyness, time_to_maturity, volatility)
-
-        spot = s.exp() * self.strike
-        delta = npdf(d2(s, t, v)) / (spot * v * t.sqrt())
-        delta = -delta if not self.call else delta  # put-call parity
-
-        return delta
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
+        return bs_european_binary_delta(
+            log_moneyness=log_moneyness,
+            time_to_maturity=time_to_maturity,
+            volatility=volatility,
+            call=self.call,
+            strike=self.strike,
+        )
 
     def gamma(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns gamma of the derivative.
 
         Args:
-            log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)`
@@ -167,24 +206,37 @@ class BSEuropeanBinaryOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
         """
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
         # TODO(simaki): Directly compute gamma.
         return super().gamma(
-            strike=self.strike,
             log_moneyness=log_moneyness,
             time_to_maturity=time_to_maturity,
             volatility=volatility,
+            strike=self.strike,
         )
 
     def vega(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns vega of the derivative.
 
         Args:
-            log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)`
@@ -194,24 +246,37 @@ class BSEuropeanBinaryOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
         """
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
         # TODO: Directly compute theta.
         return super().vega(
-            strike=self.strike,
             log_moneyness=log_moneyness,
             time_to_maturity=time_to_maturity,
             volatility=volatility,
+            strike=self.strike,
         )
 
     def theta(
-        self, log_moneyness: Tensor, time_to_maturity: Tensor, volatility: Tensor
+        self,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        volatility: Optional[Tensor] = None,
     ) -> Tensor:
         """Returns theta of the derivative.
 
         Args:
-            log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
-            volatility (torch.Tensor): Volatility of the underlying asset.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
+            volatility (torch.Tensor, optional): Volatility of the underlying asset.
 
         Shape:
             - log_moneyness: :math:`(N, *)`
@@ -224,37 +289,60 @@ class BSEuropeanBinaryOption(BSModuleMixin):
 
         Returns:
             torch.Tensor
+
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
         """
+        (
+            log_moneyness,
+            time_to_maturity,
+            volatility,
+        ) = acquire_params_from_derivative_1(
+            self.derivative, log_moneyness, time_to_maturity, volatility
+        )
         # TODO: Directly compute theta.
         return super().theta(
-            strike=self.strike,
             log_moneyness=log_moneyness,
             time_to_maturity=time_to_maturity,
             volatility=volatility,
+            strike=self.strike,
         )
 
     def implied_volatility(
         self,
-        log_moneyness: Tensor,
-        time_to_maturity: Tensor,
-        price: Tensor,
+        log_moneyness: Optional[Tensor] = None,
+        time_to_maturity: Optional[Tensor] = None,
+        price: Optional[Tensor] = None,
         precision: float = 1e-6,
     ) -> Tensor:
         """Returns implied volatility of the derivative.
 
         Args:
-            log_moneyness (torch.Tensor): Log moneyness of the underlying asset.
-            time_to_maturity (torch.Tensor): Time to expiry of the option.
+            log_moneyness (torch.Tensor, optional): Log moneyness of the underlying asset.
+            time_to_maturity (torch.Tensor, optional): Time to expiry of the option.
             price (torch.Tensor): Price of the derivative.
+            precision (float): Computational precision of the
+                implied volatility.
 
         Shape:
             - log_moneyness: :math:`(N, *)`
             - time_to_maturity: :math:`(N, *)`
+            - price: :math:`(N, *)`
             - output: :math:`(N, *)`
 
         Returns:
             torch.Tensor
+        Note:
+            args are not optional if it doesn't accept derivative in this initialization.
+            price seems optional in typing, but it isn't. It is set for the compatibility to the previous versions.
         """
+        (log_moneyness, time_to_maturity) = acquire_params_from_derivative_0(
+            self.derivative, log_moneyness, time_to_maturity
+        )
+        if price is None:
+            raise ValueError(
+                "price is required in this method. None is set only for compatibility to the previous versions."
+            )
         return find_implied_volatility(
             self.price,
             price=price,
@@ -264,6 +352,9 @@ class BSEuropeanBinaryOption(BSModuleMixin):
         )
 
 
+factory = BlackScholesModuleFactory()
+factory.register_module("EuropeanBinaryOption", BSEuropeanBinaryOption)
+
 # Assign docstrings so they appear in Sphinx documentation
 _set_docstring(BSEuropeanBinaryOption, "inputs", BSModuleMixin.inputs)
 _set_attr_and_docstring(BSEuropeanBinaryOption, "forward", BSModuleMixin.forward)
