@@ -1,3 +1,4 @@
+from typing import Any
 from typing import Callable
 from typing import List
 from typing import Optional
@@ -47,7 +48,8 @@ class Hedger(Module):
             :math:`H` is the number of hedging instruments.
         inputs (list[str|Feature]): List of the names of the input features that
             will be fed to the model.
-            See ``list(map(str, pfhedge.features.FEATURES))`` for valid options.
+            See :func:`pfhedge.features.list_feature_names` for available feature names
+            and see :ref:`features` for the details of features.
         criterion (HedgeLoss, default=EntropicRiskMeasure()):
             Loss function to minimize by hedging.
             Default: :class:`pfhedge.nn.EntropicRiskMeasure()` .
@@ -539,7 +541,7 @@ class Hedger(Module):
         """
         optimizer = self._configure_optimizer(derivative, optimizer)
 
-        def compute_loss(**kwargs) -> Tensor:
+        def compute_loss(**kwargs: Any) -> Tensor:
             return self.compute_loss(
                 derivative,
                 hedge=hedge,
